package svc

import (
	"applet/app/admin/svc/enterprise_manage"
	"applet/app/customer/md"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/enum"
	"applet/app/utils"
	"errors"
	"time"
)

func OrderList(req md.CentralKitchenForSchoolOrderListReq) (m []model.CentralKitchenForSchoolPackageOrd, total int64, err error) {
	sess := db.Db.Desc("id").Where("user_identity_id =?", req.UserIdentityId).Limit(req.Limit, (req.Page-1)*req.Limit)
	if req.OrdState != 0 {
		sess.And("ord_state = ?", req.OrdState)
	}
	total, err = sess.FindAndCount(&m)
	if err != nil {
		return
	}
	return
}

func CentralKitchenForSchoolOrderRefund(req md.CentralKitchenForSchoolOrderRefundReq) (err error) {
	sysCfgDb := db.SysCfgDb{}
	sysCfgDb.Set()
	cfg, err := sysCfgDb.SysCfgGetOne(enum.CentralKitchenForSchoolReserveMealTime)
	if err != nil {
		return
	}

	centralKitchenForSchoolUserWithDayDb := db.CentralKitchenForSchoolUserWithDayDb{}
	centralKitchenForSchoolUserWithDayDb.Set(0)
	centralKitchenForSchoolUserWithDay, err := centralKitchenForSchoolUserWithDayDb.GetCentralKitchenForSchoolUserWithDay(req.Id)
	if err != nil {
		return err
	}
	if centralKitchenForSchoolUserWithDay == nil {
		err = errors.New("未查询到订餐记录")
		return
	}
	//1、判断是否为“待就餐”状态
	if centralKitchenForSchoolUserWithDay.State != enum.CentralKitchenForSchoolUserWithDayStateForWait {
		err = errors.New("当前订餐不可退订")
		return
	}

	//2、判断是否为可退餐时间段
	now := time.Now()
	today, _ := time.Parse("2006-01-02", time.Now().Format("2006-01-02"))
	centralKitchenForSchoolReserveMealTime, _ := time.Parse("2006-01-02 15:04:05", now.Format("2006-01-02")+" "+cfg.Val+":00")
	date, _ := time.Parse("2006-01-02", centralKitchenForSchoolUserWithDay.Date)
	if today.Equal(date) {
		//2.2、判断是否过了今日可订餐时间
		if now.After(centralKitchenForSchoolReserveMealTime) {
			err = errors.New("当前订餐不在可退订时间范围内")
			return
		}
	}

	//3、处理订餐记录状态
	centralKitchenForSchoolUserWithDay.State = enum.CentralKitchenForSchoolUserWithDayStateForCanceling
	updateAck, err := centralKitchenForSchoolUserWithDayDb.CentralKitchenForSchoolUserWithDayUpdate(centralKitchenForSchoolUserWithDay.Id, centralKitchenForSchoolUserWithDay, "state")
	if err != nil {
		return err
	}
	if updateAck <= 0 {
		err = errors.New("更新订餐记录状态失败")
		return
	}

	//4、处理订单状态
	err = svc.JudgePackageOrdOrdState(req.OutTradeNo)
	if err != nil {
		return err
	}

	//5、新增退款记录 central_kitchen_for_school_user_refund_day
	outRequestNo := utils.OrderUUID(centralKitchenForSchoolUserWithDay.Uid)
	centralKitchenForSchoolUserRefundDayDb := db.CentralKitchenForSchoolUserRefundDayDb{}
	centralKitchenForSchoolUserRefundDayDb.Set(0)
	_, err = centralKitchenForSchoolUserRefundDayDb.CentralKitchenForSchoolUserRefundDayInsert(&model.CentralKitchenForSchoolUserRefundDay{
		OutTradeNo:   req.OutTradeNo,
		OutRequestNo: outRequestNo,
		Uid:          centralKitchenForSchoolUserWithDay.Uid,
		IdentityId:   centralKitchenForSchoolUserWithDay.IdentityId,
		RecordsId:    centralKitchenForSchoolUserWithDay.Id,
		State:        enum.CentralKitchenForSchoolUserRefundDayStateForAuditing,
		Amount:       centralKitchenForSchoolUserWithDay.Amount,
		Memo:         "",
		CreateAt:     now,
		UpdateAt:     now,
	})
	return
}

func CentralKitchenForSchoolOrderRefundList(req md.CentralKitchenForSchoolOrderRefundListReq) (resp []md.CentralKitchenForSchoolOrderRefundListResp, total int64, err error) {
	var m []*db.CentralKitchenForSchoolUserRefundDayWithCentralKitchenForSchoolUserWithDay
	sess := db.Db.Where("central_kitchen_for_school_user_refund_day.identity_id =?", req.UserIdentityId)
	total, err = sess.
		Join("LEFT", "central_kitchen_for_school_user_with_day", "central_kitchen_for_school_user_refund_day.records_id = central_kitchen_for_school_user_with_day.id").
		Limit(req.Limit, (req.Page-1)*req.Limit).FindAndCount(&m)
	if err != nil {
		return nil, 0, err
	}

	for _, v := range m {
		resp = append(resp, md.CentralKitchenForSchoolOrderRefundListResp{
			OutTradeNo:   v.CentralKitchenForSchoolUserRefundDay.OutTradeNo,
			OutRequestNo: v.CentralKitchenForSchoolUserRefundDay.OutRequestNo,
			Kind:         v.CentralKitchenForSchoolUserWithDay.Kind,
			CreateAt:     v.CentralKitchenForSchoolUserRefundDay.CreateAt.Format("2006-01-02 15:04:05"),
			Date:         v.CentralKitchenForSchoolUserWithDay.Date,
			State:        v.CentralKitchenForSchoolUserRefundDay.State,
			Memo:         v.CentralKitchenForSchoolUserRefundDay.Memo,
			Amount:       v.CentralKitchenForSchoolUserRefundDay.Amount,
		})
	}
	return
}
